@@ -1,4 +1,4 @@
-# (c) Sound Object Logic 2000-2001
+
 
 use strict;
 
@@ -90,7 +90,9 @@ sub get_sequence {
     # the only database that has a non-trivial sequence sql fragment
     # also doesn't use " FROM DUAL"
     my $query = $self->sequence_sql($sequence_name).$self->from_dual;
-    my ($id) = map { @$_ } $self->{db}->selectall_arrayref($query);
+    my ($id) = (map { @{$_} }
+		map { @{$_} }
+		$self->{db}->selectall_arrayref($query));
 
     return $id;
 }
@@ -102,6 +104,13 @@ sub sequence_sql
 	return $self->{driver}->sequence_sql(shift);
     }
 
+sub limit_sql {
+    my $self = shift;
+
+    my $driver = $self->{driver} or confess "no driver";
+    return $self->{driver}->limit_sql(@_);
+}
+
 sub _open
   {
     my ($self, $schema) = @_;
@@ -174,7 +183,10 @@ sub _open
 
     $self->{get_id} = $schema->{get_id} || sub {
 	  my $obj = shift or warn "no object passed to get_id";
-	  my $address = Tangram::refaddr($obj);
+	  ref $obj or return undef;
+	  my $address = Tangram::refaddr($obj)
+	      or do { warn "Object $obj has no refaddr(?)";
+		      return undef };
 	  my $id = $self->{ids}{$address};
 	  if ($Tangram::TRACE && ($Tangram::DEBUG_LEVEL > 2)) {
 		print $Tangram::TRACE "Tangram: $obj is ".($id?"oid $id" : "not in storage")."\n";
@@ -306,7 +318,8 @@ sub make_id
 	$id = $classdef->{make_id}->($class_id, $self);
 	print $Tangram::TRACE "Tangram: custom per-class ($cname) make ID function returned ".(pretty($id))."\n" if $Tangram::TRACE;
     } elsif ( $classdef->{oid_sequence} ) {
-	$id = $self->get_sequence($classdef->{oid_sequence});
+	eval { $id = $self->get_sequence($classdef->{oid_sequence}) };
+	die "Failed to get sequence for Class `$cname'; $@" if $@;
     }
 
     # maybe the entire schema has its own ID generator
@@ -314,8 +327,11 @@ sub make_id
 	$id = $self->{schema}{sql}{make_id}->($class_id, $self);
 	print $Tangram::TRACE "Tangram: custom schema make ID function returned "
 	    .(pretty($id))."\n" if $Tangram::TRACE;
-    } elsif ( my $seq = $self->{schema}{sql}{oid_sequence} ) {
-	$id = $self->get_sequence($seq);
+    } elsif ( !defined($id) &&
+	      (my $seq = $self->{schema}{sql}{oid_sequence}) ) {
+	eval { $id = $self->get_sequence($seq) };
+	die "Failed to get sequence for Class `$cname' via fallback $seq; $@"
+	    if $@;
     }
     if (defined($id)) {
 	return $self->combine_ids($id, $class_id);
@@ -412,7 +428,9 @@ sub class_id
 	my @stack = \%{$_[0]."::"};
 	my $seen = Set::Object->new(@stack);
 	while ( my $stash = pop @stack ) {
-	    my @supers = @{ *{$stash->{ISA}}{ARRAY} };
+            defined $stash or next;
+            my @supers = @{ *{$stash->{ISA}}{ARRAY} }
+		if exists $stash->{ISA};
 	    for my $super ( @supers ) {
 		if ( defined $self->{class2id}{$super} ) {
 		    $self->{class2id}{$_[0]}
@@ -1108,8 +1126,13 @@ sub count
     }
     else
     {
-	my $expr = shift;
-	$target = $expr->{expr};
+	my $expr = shift or croak "nothing supplied to count";
+	if ($expr->isa("Tangram::QueryObject")) {
+	    $target = "*";
+	    $expr = $expr->{id};
+	} else {
+	    $target = $expr->{expr};
+	}
 	$objects->insert($expr->objects);
 	$filter = shift;
     }
@@ -1174,10 +1197,29 @@ sub sum
 sub id
 {
     my $self = shift;
-	return map { $self->{get_id}->($_) } @_ if wantarray;
+    return map { $self->{get_id}->($_) } @_ if wantarray;
     $self->{get_id}->(shift());
 }
 
+sub id_maybe_insert
+{
+    my $self = shift;
+    return map { scalar($self->id_maybe_insert($_)) }
+	@_ if wantarray;
+
+    my $object = shift;
+    if ( my $id = $self->{get_id}->($object) ) {
+	return $id;
+    } else {
+	my $class = ref $object;
+	if ( eval { $self->class_id($class) } ) {
+	    print $Tangram::TRACE "id_maybe_insert: inserting $object\n"
+		if $Tangram::TRACE;
+	    return $self->insert($object);
+	}
+    }
+}
+
 sub disconnect
 {
     my ($self) = @_;
